/* a set of routines for transfering data between the Psion and the Sun machines
   where possible I have provided a sensible wrapper for the low level function
   This program should be started BEFORE the suncom program on the psion */
#include <stdio.h>
#include <ctype.h>
#include <sys/termios.h>
#include <fcntl.h>
#include <string.h>
#include "psion.h"
#define MAXSEND 200
#define DEFVSNSTR "1.11"

static char vsn[]= "@(#) pslib.c 1.11@(#)" ;
char pspart[10] = "" ;
char psdir [255] = "";
char psfname[30] = "";
int pspartset, psdirset, psfnameset ;
int tmode ;
int transct ;
int debuglink = FALSE ;

char version[100] ="X" ;

int devid;
struct termios initial, modified ;

psionparse(psionstr, pp, pd, pf)
char * psionstr, * pp, * pd, * pf ;
{
	char * tmp ;

	/* this routine assumes a directory name exisit (not unreasonable with a psion */
	/* first remove the LOC:: from the string */
	psionstr += 5 ;
	/* (Simple Hu !) */
	/* now extract the next two characters which will be drive name and :*/
	pp[0] = psionstr[0] ;
	pp[1] = psionstr[1] ;
	pp[2] = '\0' ;

	/* increment psionstr to remove the drive spec  and leading \ from the filenames */
	psionstr += 3 ;
	/* isnt pointer arithmetic fun ! */
	/* now find the last \ in the remaining name, this will be the
	end of the directory string */
	tmp = strrchr(psionstr, '\\') ;
	/* terminate the directory name */
	tmp[0] = '\0' ;
	/* tmp will include the \ (now '\0') so increment by one */
	tmp ++ ;
	/* extract the filename from psionstr using tmp */
	strcpy (pf, tmp) ;
	/* now the dir name which will be null terminated (see above) */
	strcpy (pd, psionstr) ;
}

psisset(qn)
int qn ;
{
	if (qn == PART)
		return(pspartset) ;
	else if (qn == DIR)
		return(psdirset) ;
	else if (qn == FNAME) 
		return(psfnameset) ;
	else
		return (BADPARAM) ;
}

pstrans()
{
	return(transct) ;
}

psinit(restart, devname)
int restart ;
char * devname ;
{
   devid = 0 ;
   tmode = BINARY ;
   psinitlink(devname) ;
   /* is we are doing a restart we send the online cmd by hand
      WITHOUT waiting for any fancy XCMD> prompts to come back
      to us */
   if (restart)
	psrestart() ;
   else
   	psreply("XONLINE") ;
   psbinary() ;
}

psrestart()
{
	char length[5] ;

	/* cheat the system for the restart mode only. to have got here 
	   we know that the link opened correctley */
	write(devid, "$02ol",5) ;
	psreply("XONLINE") ;
	psup() ;
}

psinitlink(devstr) 
char * devstr ;
{
	if (strlen(devstr) == 0)
	{
		if ((devid = open("/dev/ttya", O_RDWR , 0)) == -1)
			pserror ("Error in opening device\n") ;
	}
	else
	{
		if ((devid = open(devstr, O_RDWR, 0)) == -1)
			pserror ("Error in openint device\n") ;
	}

	/* we need to remove the echo as it confuses the psion (tremendously!) */
	/* first get the current structure for storage and modification*/
	ioctl(devid, TCGETS, &initial);
	ioctl(devid, TCGETS, &modified);
	/* remove the echo line */
	modified.c_lflag = modified.c_lflag & ~ECHO ;
	/* stop it from doing any flay input processing */
	modified.c_lflag = modified.c_lflag & ~IEXTEN ;
	/* setup the input and output speeds */
	modified.c_cflag = B9600 | CS8 | CREAD | ( B9600 << IBSHIFT ) ;
	/* remove incomming parity checks */
	modified.c_iflag = modified.c_iflag & ~INPCK ;
	/* remove CR on input, make life a bit easier on the input routing coding ! */
	modified.c_iflag = modified.c_iflag | IGNCR ; 
	/* strip down to 7 characters */
	modified.c_iflag = modified.c_iflag | ISTRIP ;
	/* Install xon/xoff flow control */
	/*modified.c_iflag = modified.c_iflag | IXON | IXOFF ;*/
	/* remove flow control */
	modified.c_iflag = modified.c_iflag & ~IXON ;
	modified.c_iflag = modified.c_iflag & ~IXOFF ;
	modified.c_cflag = modified.c_cflag & ~CRTSCTS ;
	/* ensure that on output we only send newline, not CRNL */
	modified.c_oflag = modified.c_oflag | ONLRET ;
	/* and just to make sure remove the output processing facility */
	modified.c_oflag = modified.c_oflag & ~OPOST ;
	/* set the new params */
	ioctl(devid, TCSETS, &modified) ;

}

/* reset the terminal device and close the link down */
psshutdown(ret)
int ret ;
{
	/* we need to reset the terminal flags to their initial state ! */
	ioctl (devid, TCSETS, &initial) ;

	if (devid != -1)
		close (devid) ;

	exit(ret) ;
}

/* send a command to the psion */
pscommand (msg)
char * msg ;
{
	psputline(msg) ;
}

/* send some text to the psion */
psputline1 (str,len)
char * str ;
int len ;
{
	char length[5] ;
	psresponse(">") ;
	/* New method - send the length in 3 ascii bytes then send the 
	   remainder of the text in one fell swoop, no terminator etc */
	sprintf(length, "$%2.2X", len) ;
	write(devid, length, 3) ;
	write(devid, str, strlen(str)) ;

	if (debuglink)
		printf("Len %s, Sent :%s:\n", length, str) ;
}

psputline(str)
char * str;
{
	psputline1(str, strlen(str)) ;
}

/* get a line of text from the psion, remove any newlines etc */
psresponse(str)
char * str ;
{
	psreadstr(str, 1000) ;
	if (str[strlen(str)] == '\n') ;
	   str[strlen(str)] == '\0' ;

	if (debuglink)
		printf("Len %d, Got:%s:\n", strlen(str), str) ;
}

/* get a line of text from the psion and remove any terminating newlines */
psreadstr(string, count)
char * string ;
int count ;
{
	int ctr ;
	ctr = 0 ;
	ctr = read(devid, string, count) ;
	string[ctr -1] = '\0' ;
	if (string[ctr - 1] == '\n') ;
	   string[ctr - 1] == '\0' ; 
	/*   New method, the psion sends a string length, two hex charachers
	   and then the string in hex, no newlines etc come into play*/
	/*
	int ctr, cnt;
	char cntstr[10] ;
	ctr = read(devid, cntstr, 2) ;
	cnt = dehex(cntstr) ;
	ctr = read(devid, string,  */
}

/* send a file to the psion, remove the file if it exists. if we have been 
   passed a null string use the file name we are sending as the local files name
   */
pssendfile(fname)
char * fname ;
{
	/* first remove the target file is required */
	psclearfile() ;
	if (strlen(fname) == 0)
		psgf(psfname) ;
	else
		psgf(fname) ;
	/* the above command is misnamed as if refers to getting a file on the psion ! */
}

/* we have an error, print the message ans shutdown */
pserror(str)
char * str ;
{
	fprintf(stderr, "%s\n", str) ;
	psshutdown(6) ;
}

psgf(fname)
char * fname ;
{
	if (tmode == ASCII)
		psgfa(fname) ;
	else
		psgfb(fname) ;
}

/* send the file named in the parameter to the psion. this assumes that the 
   file does not exist on the psion and that the remote path is correct
   then send a hex file */

psgfb(fname)
char * fname ;
{
	FILE *fd ;
	char sendstr [255], tmpstr [10] ;
	char ch ;
	int ctr ;
	int fileid ;

	transct = 0 ;

	if ((fileid = open(fname, O_RDONLY , 0)) == -1)
	{
	   pserror ("File not found, psgf\n") ;
	   psshutdown (1) ;
	}

	pscommand("gf") ;

	psreply("XSTARTGET") ;
	if (psquery("XBADOPEN"))
	   return ;

	ctr = 0 ;
	sendstr[0] = '\0' ;

	while (read(fileid, &ch, 1) == 1)
	{
	   pstoas(ch, tmpstr) ;
	   strcat(sendstr, tmpstr) ;
	   ctr += 2 ;
	   transct ++ ;
	   if (ctr > MAXSEND)
	   {
	      psputline(sendstr) ;
	      ctr = 0 ;
	      sendstr[0] = '\0' ;
	   }
	}
	/* clear out any data in the string */
	if (ctr != 0)
	{
	   psputline(sendstr) ;
	}

	psputline("XE") ;
	psquery("XENDGET") ;
	close(fd) ;
}

/* send the file named in the parameter to the psion. this assumes that the 
   file does not exist on the psion and that the remote path is correct
   then send a text file */

psgfa(fname)
char * fname ;
{
	FILE *fd ;
	char sendstr [255], tmpstr [10] ;
	char ch ;
	int ctr ;
	int fileid ;

	transct = 0  ;

	if ((fileid = open(fname, O_RDONLY , 0)) == -1)
	{
	   pserror ("File not found, psgf\n") ;
	   psshutdown (1) ;
	}

	pscommand("gf") ;

	psreply("XSTARTGET") ;
	if (psquery("XBADOPEN"))
	   return ;

	ctr = 0 ;
	sendstr[0] = '\0' ;

	while (read(fileid, &ch, 1) == 1)
	{
	   sendstr[ctr] = ch ;
	   ctr += 1 ;
	   transct ++ ;
	   if (ctr > MAXSEND)
	   {
	      psputline1(sendstr, ctr) ;
	      ctr = 0 ;
	      sendstr[0] = '\0' ;
	   }
	}
	/* clear out any data in the string */
	if (ctr != 0)
	{
	   psputline1(sendstr, ctr) ;
	}

	psputline("XE") ;
	psquery("XENDGET") ;
	close(fd) ;
}


pstoas(ch, str) 
char ch ;
char * str ;
{
	int msn, lsn ;
	int tmp ;
	tmp = ch & 0xFF ;
	msn = ((int) tmp) / 16 ;
	lsn = (int) tmp - (msn * 16) ;
	str[0] = 'A' + msn ;
	str[1] = 'A' + lsn ;
	str[2] = '\0' ;
}

psrecfile(fname)
char * fname ;
{
	/* if the file exists remotley */
	if (psef())
	/* check to see if fname is null, if so use the system file name */
		if (strlen(fname) == 0)
			pspf(psfname) ;
		else
			pspf(fname) ;
}

/* call the appropriate transfer method */
pspf(fname)
char * fname ;
{
	if (tmode == ASCII)
		pspfa(fname) ;
	else
		pspfb(fname) ;
}

pspfa(fname)
char * fname ;
{
	FILE *fd ;
	char recstr [255], tmpstr [10] ;
	char ch, dehex() ;
	int fileid,ctr, len ;

	transct = 0 ;

	if ((fileid = open(fname, O_WRONLY|O_CREAT , 420)) == -1)
	{
	   pserror ("File not found, pspfa\n") ;
	   psshutdown (1) ;
	}

	pscommand("pf") ;

	psreply("XSTARTPUT") ;
	if (psquery("XBADOPEN"))
	   return ;

	psresponse(recstr) ;
	while (strcmp(recstr, "XE") != 0)
	{
		ctr = 0 ;
		len = strlen(recstr) ;
		while ( ctr <= (len - 2))
		{
			ch = dehex(recstr + ctr) ;
			write(fileid, &ch, 1) ;
			ctr += 2 ;
			transct ++ ;
		}
		psresponse(recstr) ;
	}

	psquery("XENDPUT") ;
	close(fileid) ;


}

pspfb(fname)
char * fname ;
{
	FILE *fd ;
	char recstr [255], tmpstr [10] ;
	char ch, dehex() ;
	int fileid, ctr, len ;

	transct = 0 ;

	if ((fileid = open(fname, O_WRONLY |O_CREAT, 420)) == -1)
	{
	   pserror ("File not found, pspfb\n") ;
	   psshutdown (1) ;
	}

	pscommand("pf") ;

	psreply("XSTARTPUT") ;
	if (psquery("XBADOPEN"))
	   return ;

	psresponse(recstr) ;
	while (strcmp(recstr, "XE") != 0)
	{
		ctr = 0 ;
		len = strlen(recstr) ;
		while ( ctr <= (len - 2))
		{
			ch = dehex(recstr + ctr) ;
			write(fileid, &ch, 1) ;
			ctr += 2 ;
			transct ++ ;
		}
		psresponse(recstr) ;
	}

	psquery("XENDPUT") ;
	close(fileid) ;

}

char dehex (str)
char * str ;
{
   char ch ;
   int tmp ;
   char tmpstr[6] ;

   tmpstr[0] = '0' ;
   tmpstr[1] = 'x' ;
   tmpstr[2] = str[0] ;
   tmpstr[3] = str[1] ;
   tmpstr[4] = '\0' ;

   sscanf(tmpstr, "%i", &tmp) ;

   ch = (char) tmp ;
   return (ch) ;
}

psscreenoff()
{
	pscommand("is") ;
	if (psquery ("XYES") == 1)
		pssc() ;
		
}

psscreenon()
{
	pscommand("is") ;
	if (psquery ("XNO") == 1)
		pssc() ;
		
}

pssc()
{
	pscommand("sc") ;
	psreply("XOK") ;
}

psverboseoff()
{
	pscommand("iv") ;
	if (psquery ("XYES") == 1)
		psvb() ;
		
}

psverboseon()
{
	pscommand("iv") ;
	if (psquery ("XNO") == 1)
		psvb() ;
		
}

char * psversion()
{
	char vsnstr[100] ;

	/* if the first char of version is X we have not got the version
	   yet so get it and store it, otherwise just return the pointer
	   to the version string */
	if (version[0] == 'X')
	{
		psvn(vsnstr) ;
		/* copy from the second char as the first will be X */
		strcpy(version, &vsnstr[1]) ;
		/* if the first char is B (the psion will return XBADCMD
		   if it doesnot support version checking yet) run pssetdefvsn */
		if (version[0] == 'B')
			pssetdefvsn() ;
	}
	return (version) ;
}

/* compare psion vsn string to sun vsn string, if the psion one is later return
   GREATERTHAN, if they are the same EQUAL otherwise LESSTHAN */
psvsncmp(vsstr)
char * vsstr ;
{
	char * psvnstr ;
	char * ps1, *ps2, *ps3, *ps4 ;
	char * ss1, *ss2, *ss3, *ss4 ;
	char nstr[1] ;
	int pint, vint ;

	/* first ensure that we have the correct version */
	psvnstr = psversion() ;

	/* initialise the nstr */
	nstr[0] = '\0' ;

	/* now extract the strings */
	ps1 = psvnstr ;
	ss1 = vsstr ;

	/* locate the next . */
	psvnstr = strchr(psvnstr, '.') ;
	vsstr = strchr(vsstr, '.') ;
	if (psvnstr == NULL)
		psvnstr = nstr ;
	else
		psvnstr ++;/* skip over the . */
	if (vsstr == NULL)
		vsstr = nstr ;
	else
		vsstr ++ ; /* see above */
	
	ps2 = psvnstr;
	ss2 = vsstr ;
	/*  and the next */
	psvnstr = strchr(psvnstr, '.') ;
	vsstr = strchr(vsstr, '.') ;
	if (psvnstr == NULL)
		psvnstr = nstr ;
	else
		psvnstr ++;/* skip over the . */
	if (vsstr == NULL)
		vsstr = nstr ;
	else
		vsstr ++ ; /* see above */
	
	ps3 = psvnstr ;
	ss3 = vsstr ;
	/* and the last */
	psvnstr = strchr(psvnstr, '.') ;
	vsstr = strchr(vsstr, '.') ;
	if (psvnstr == NULL)
		psvnstr = nstr ;
	else
		psvnstr ++;/* skip over the . */
	if (vsstr == NULL)
		vsstr = nstr ;
	else
		vsstr ++ ; /* see above */

	ps4 = psvnstr ;
	ss4 = vsstr ;

	/* do the first test */
	pint = atoi(ps1) ;
	vint = atoi(ss1) ;

	if (pint > vint)
		return (GREATERTHAN) ;
	else if (pint < vint)
		return (LESSTHAN) ;
	pint = atoi(ps2) ;
	vint = atoi(ss2) ;

	if (pint > vint)
		return (GREATERTHAN) ;
	else if (pint < vint)
		return (LESSTHAN) ;
	pint = atoi(ps3) ;
	vint = atoi(ss3) ;

	if (pint > vint)
		return (GREATERTHAN) ;
	else if (pint < vint)
		return (LESSTHAN) ;
	pint = atoi(ps4) ;
	vint = atoi(ss4) ;

	if (pint > vint)
		return (GREATERTHAN) ;
	else if (pint < vint)
		return (LESSTHAN) ;

	/* they would appear to be the same */
	return (EQUAL) ;
}


psvsngt(vsn)
char * vsn ;
{
	if (psvsncmp(vsn) == GREATERTHAN)
		return (TRUE) ;
	else
		return(FALSE) ;
}

psvsnlt(vsn)
char * vsn ;
{
	if (psvsncmp(vsn) == LESSTHAN)
		return(TRUE) ;
	else
		return(FALSE) ;
}

psvsneq(vsn)
char * vsn ;
{
	if (psvsncmp(vsn) == EQUAL)
		return(TRUE) ;
	else
		return(FALSE) ;
}
pssetdefvsn()
{
	/* set the default version string */
	strcpy(version, DEFVSNSTR) ;
}

psvn(vsnstr)
char * vsnstr ;
{
	pscommand("vn") ;
	psresponse(vsnstr) ;
}

psvb()
{
	pscommand("vb") ;
	psreply("XOK") ;
}

psds()
{
	char part[100], dir[100], filename[100] ;

	pscommand("ds") ;
	psreply("XSTART") ;
	psresponse(part) ;
	psresponse(dir) ;
	psresponse(filename) ;
	psreply("XEND") ;
}

psascii()
{
	tmode = ASCII ;
	pscommand("ia") ;
	if (psquery("XNO") == 1)
		pstm() ;
}

psbinary()
{
	tmode = BINARY ;
	pscommand("ia") ;
	if (psquery("XYES") == 1)
		pstm() ;
}

pstm()
{
	pscommand("tm") ;
	psreply("XOK") ;
}
int psquery(str)
char * str ;
{
	char tmp[100] ;
	psresponse(tmp) ;
	if (strcmp(tmp, str) == 0)
	   return(TRUE) ;
	else
	   return (FALSE) ;
}

psclearfile() 
{
	if (psef() == TRUE)
		psrf() ;
}

psef()
{
	pscommand("ef") ;
	if (psquery("XYES") == TRUE)
		return(TRUE) ;
	else
		return(FALSE) ;
}


psed()
{
	pscommand("ed") ;
	return(psquery("XYES") == 1) ;
}

psep()
{
	pscommand("ep") ;
	return(psquery("XYES") == 1);
}

psrf()
{
	pscommand("rf") ;
	psreply("XOK") ;
}

psrd()
{
	pscommand("rd") ;
	psreply("XOK");
}

psmd()
{
	pscommand("md");
	psreply("XOK") ;
}

psrl()
{
	pscommand("rl") ;
	psreply("XOK") ;
}


pslf(fd)
FILE *fd ;
{
	pscommand("lf") ;
	psldir(fd) ;
}

psld(fd)
FILE *fd ;
{
	pscommand("ld") ;
	psldir(fd) ;
}

pslp(fd)
FILE *fd ;
{
	pscommand("lp") ;
	psldir(fd) ;
}
psldir(fd)
FILE *fd ;
{
	char txt[255] ;
	if (psquery("XSTARTLIST"))
		while(1)
		{
			psresponse(txt) ;
			if (strcmp (txt, "XENDLIST") == 0)
				break ;
			if (fd == NULL)
				printf("File :%s\n", txt) ;
			else
				fprintf(fd, "%s\n", txt) ;
		}
}

psdc()
{
	pscommand("dc") ;
	psreply("XDISCONNECTED") ;
	psshutdown(0) ;
}

psreply (expected)
char * expected ;
{
   char tmp[1000] ;

   psresponse(tmp) ;

   if (strcmp(tmp, expected) == 0)
	return;
   else
	psshutdown(1);
}

psdt (txt)
char * txt ;
{
	char tmp [1000] ;
	strcpy (tmp, "dt ") ;
	strcat (tmp, txt) ;
	pscommand (tmp) ;
	psreply("XOK") ;
}

pssetpath(part, dir, file)
char * part, *dir, *file ;
{
	if (strlen(part) == 0)
		pssp (pspart) ;
	else
		pssp(part) ;

	if (strlen(dir) == 0)
		pssd (psdir) ;
	else
		pssd(dir) ;

	if (strlen(file) == 0)
		pssf(psfname) ;
	else
		pssf (file) ;
}

pssp(part)
char *part ;
{
	char tmp[100] ;

	/* is the stored name is the same as the current name save on 
	   transmitting information */
	if (strcmp(pspart, part) != 0)
	{
		strcpy(tmp, "sp ") ;
		strcat(tmp, part) ;

		pscommand(tmp) ;
		psreply ("XOK") ;

		strcpy(pspart, part) ;
	} /* we will have to reset these two anyway */
	strcpy(psdir, "") ;
	strcpy(psfname, "") ;
}

psup()
{
	pscommand("up") ;
	psreply("XOK") ;
	strcpy(pspart, "") ;
	strcpy(psdir, "") ;
	strcpy(psfname, "") ;
}

pssd(dir)
char * dir ;
{
	char tmp[100] ;

	/* if the stored name is the same as the current name save on data
	   transfer */
	if (strcmp(psdir, dir) != 0)
	{
		strcpy(tmp, "sd ") ;
		strcat(tmp, dir) ;

		pscommand(tmp) ;
		psreply ("XOK") ;
		strcpy(psdir, dir) ;
	}
	/* we will still have to reset the file name */
	strcpy(psfname, "") ;
}

psud()
{
	pscommand("ud") ;
	psreply("XOK") ;
	/* unset the internal file and dir names */
	strcpy(psdir, "") ;
	strcpy(psfname, "") ;
}

pssf(filename)
char * filename ;
{
	char tmp[100] ;

	/* if the store file name and the new are the same ignore the transmit */
	if (strcmp(psfname, filename) != 0)
	{
		strcpy(tmp, "sf ") ;
		strcat(tmp, filename) ;

		pscommand(tmp) ;
		psreply ("XOK") ;
		/* store the file name for internal use */
		strcpy(psfname, filename) ;
	}
}

psuf()
{
	pscommand("uf") ;
	psreply("XOK") ;
	/* unset the internal file name */
	strcpy(psfname, "") ;
}

ispartset()
{
	return(strlen(pspart)) ;
}

