/****************************************************************************/
/*                 Routine to test xmodem with serial commun on HC          */
/*           Written for HC                                                 */
/* Author:   Michael Fappiano      (c) Psion Inc. 1991                      */
/* File:     XMOD.C                                                         */
/*                                                                          */
/* Sep-30-91  MDF     Created                                               */
/*                                                                          */
/* Requires xmod.dat on default directory.  This is just a sample data file.*/
/* Creates recv.dat on default directory.                                   */
/* link should not be running on HC                                         */
/****************************************************************************/
#include <plib.h>
#include <p_serial.h>  /* has P_SRCHAR type def */
#include <p_xmodem.h>
#include <p_cons.h>  /* used to set console size */
#define  BUFSIZ      128  /* must be 128 for XMODEM */

GLREF_D VOID *winHandle;
VOID errhand(VOID);


int main(void) {

    VOID    *pcb, *fcb;
    INT     io_stat;
    TEXT    out_buff[BUFSIZ];  /* send buffer */
    INT     obuff_len;     /* size of output buffer */
    P_SRCHAR    srchar;
    INT         zero;          /* constant 0 */
    WORD        status;
    UWORD       mode, error;
    INT         blkcnt;
    P_RECT      rect;
    P_POINT     point;
    INT         func;

     /* get the window in order */
   io_stat=p_open(&winHandle,"CON:",0);
   if (io_stat<0) {
      p_printf("Window error %d", io_stat);
      errhand();
   }
   rect.tl.x = rect.tl.y = 0;
   rect.br.x = 25;
   rect.br.y = 9;
   func = P_SCR_WSET;
   p_iow4(winHandle,P_FSET,&func,&rect);
   func = P_SCR_CLR;  /* clear screen */
   p_iow4(winHandle,P_FSET,&func,&rect);

    zero=0;
    p_srep(out_buff, BUFSIZ, "*");   /* init the buffer to "*** ... ***" */
                                      /* done as a debugging tool */
    obuff_len = BUFSIZ;

/*************************************
 *   Transmit from HC                *
 *************************************/
    p_printf("Begin xmit test.");
    /* open the  serial port */
    io_stat = p_open(&pcb, "TTY:A", -1); /* open the serial port */
    if (io_stat<0)
      io_stat = p_open(&pcb, "TTY:B", -1); /* open the serial port */
    p_printf("Open status=%d", io_stat);
    if (io_stat<0)
       errhand();

    srchar.tbaud = srchar.rbaud = P_BAUD_9600; /* serial protocol */
    srchar.hand = P_IGN_CTS;  /* no handshaking */

    io_stat = p_iow(pcb, P_FSET, &srchar);
    p_printf("Set status=%d", io_stat);
    if (io_stat<0)
       errhand();

    io_stat = p_open(&pcb, "XMD:", -1); /* attach XMODEM drivers */
    p_printf("Open status=%d", io_stat);
    if (io_stat<0)
       errhand();

/* connect to remote device */
    mode=P_XMDM_ACCP;     /* wait for remote end to initiate link */
                          /* use this mode for sending from the HC */
    error=P_XMDM_CRCORCHECKSUM;       /* CRC or CHECKSUM */
    io_stat=p_ioa(pcb,P_FCONNECT,&status,&mode,&error);
    p_printf("Starting connect %d", io_stat);
    if (io_stat<0)
       errhand();

/* wait for connection */
    while (status==E_FILE_PENDING)
      p_print("*");
    p_iowait(); /* use up the signal generated by the ioa */

    io_stat=status;
    p_printf("Connect status %d", io_stat);
    if (io_stat<0)
      errhand();


    p_printf("Output test");
    /* open file */
    io_stat=p_open(&fcb,"xmod.dat",P_FOPEN|P_FSTREAM_TEXT|P_FSHARE);
    p_printf("Open file status %d", io_stat);
    if (io_stat<0)
      errhand();

    blkcnt=0;  /* block count */

    p_printf("");
    func = P_SCR_POSA;
    point.x = 0;
    point.y = 9;  /* bottom line of HC */
    p_iow4(winHandle,P_FSET,&func,&point);   /* move cursor */
    p_print("Sending block ");

    FOREVER {
      io_stat = p_read(fcb, out_buff, obuff_len);
      if (io_stat==E_FILE_EOF)
        break;
      if (io_stat<0) {
         p_printf("Error reading file %d", io_stat);
         errhand();
      }

      io_stat = p_iow(pcb, P_FWRITE, out_buff, &obuff_len);
      if (io_stat<0) {
         p_printf("Error Xmod write %d", io_stat);
         errhand();
      }
      point.x = 15;
      point.y = 9;
      func = P_SCR_POSA;
      p_iow4(winHandle,P_FSET,&func,&point);
      p_print("%d", blkcnt);
      blkcnt++;
    }

    point.x = 15;
    point.y = 9;
    func = P_SCR_POSA;
    p_iow4(winHandle,P_FSET,&func,&point);  /* move cursor */
    p_printf("%d", blkcnt);
    p_printf("Send end of file marker");
    io_stat = p_iow(pcb, P_FWRITE, out_buff, &zero);
    p_printf("EOF send status %d", io_stat);
    if (io_stat<0)
       errhand();

    /* close the  output file */
    p_close(fcb);

    /* disconnect from XMODEM device */
    io_stat=p_iow(pcb, P_FDISCONNECT, zero, zero);
    p_printf("DISCONNECT status %d", io_stat);
    if (io_stat<0)
       errhand();

/*************************************
 *   Receive on HC                   *
 *************************************/

    p_printf("Input test");
    p_printf("Press key");
    p_getch();

 /* reads back data from a file and dumps it a file */
     /* open input file */
    io_stat=p_open(&fcb,"recv.dat",P_FREPLACE|P_FSTREAM_TEXT|P_FUPDATE);
    p_printf("Open file status %d", io_stat);
    if (io_stat<0)
      errhand();

/* connect to remote device */

    mode=P_XMDM_INIT;     /* HC will initiate link */
                          /* use this mode for receiving on the HC */
    error=P_XMDM_CRCORCHECKSUM;       /* CRC or CHECKSUM */
    io_stat=p_ioa(pcb,P_FCONNECT,&status,&mode,&error);
    p_printf("Starting connect %d", io_stat);
    if (io_stat<0)
       errhand();

/* wait for connection */
    while (status==E_FILE_PENDING)
      p_print("*");
    p_iowait(); /* use up the signal generated by the ioa */

    io_stat=status;
    p_printf("Connect status %d", io_stat);
    if (io_stat<0)
      errhand();

    blkcnt=0;

    p_printf("");
    func = P_SCR_POSA;
    point.x = 0;
    point.y = 9;  /* bottom line of HC */
    p_iow4(winHandle,P_FSET,&func,&point);   /* move cursor */
    p_print("Receiving block ");

    FOREVER {
      io_stat = p_read(pcb, out_buff, obuff_len);
      if (io_stat==0) /* end of transmission received */
        break;

      if (io_stat<0) {
         p_printf("Error receiving data %d", io_stat);
         errhand();
      }

      /* to store data to a file, do a p_write
         to the file with the data in out_buff */
      io_stat = p_write(fcb, out_buff, obuff_len);
      if (io_stat<0) {
         p_printf("error writing to recv %d", io_stat);
         errhand();
      }
      point.x = 16;
      point.y = 9;
      func = P_SCR_POSA;
      p_iow4(winHandle,P_FSET,&func,&point); /* move cursor */
      p_print("%d", blkcnt);
      blkcnt++;
    }

    point.x = 16;
    point.y = 9;
    func = P_SCR_POSA;
    p_iow4(winHandle,P_FSET,&func,&point);  /* move cursor */
    p_printf("%d", blkcnt);

    p_printf("");

    p_close(pcb);      /* close serial port */
    p_close(fcb);      /* close file */
    p_print("Demo completed.");
    p_print("Press key.");
    p_getch();
    return(io_stat);
}

/****************************************************************************/
/*                 Routine to stop program on error                         */
/*           Written for HC                                                 */
/* File:     XMOD.C                                                         */
/*                                                                          */
/* Sep-30-91  MDF     Created                                               */
/*                                                                          */
/****************************************************************************/
VOID errhand(VOID) {
     p_printf("***ERROR***");
     p_getch();   /* let the system clean up */
     p_exit(-1);
}

